import 'dart:async';

import 'package:dartz/dartz.dart';
import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/exceptions.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/datasources/user_local_data_source.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/datasources/user_remote_data_source.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_accept_enrollment_form_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_get_enrollment_form_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/data/models/user_login_response.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/legal_document.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/user.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/repositories/user_repository.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/enum_helper.dart';
import 'package:meta/meta.dart';

class UserRepositoryImpl implements UserRepository {
  final UserRemoteDataSource remoteDataSource;
  final UserLocalDataSource localDataSource;

  UserRepositoryImpl({
    @required this.remoteDataSource,
    @required this.localDataSource,
  });

  @override
  Future<Either<Failure, UserLoginResponse>> logIn({
    @required String externalId,
    @required User user,
  }) async {
    try {
      final failureOrUserLoginResponse = await remoteDataSource.logIn(
        externalId: externalId,
        user: user,
      );

      if (failureOrUserLoginResponse.isRight()) {
        final userLoginResponse = failureOrUserLoginResponse.getOrElse(null);
        ConfigReader.setUserJwt(userLoginResponse.sessionToken);
        if (userLoginResponse.legalTermsAccepted) {
          final jwt = userLoginResponse.sessionToken;
          final clientType = userLoginResponse.clientType;
          await localDataSource.cacheJwt(jwt);
          ConfigReader.setClientType(clientType);
          await localDataSource
              .cacheClientType(EnumHelper.enumToString(clientType));
        }
      }

      return failureOrUserLoginResponse;
    } on ServerException {
      return Left(ServerFailure());
    }
  }

  @override
  Future<Either<Failure, bool>> isLoggedIn() async {
    try {
      final jwt = await localDataSource.getJwt();
      if (jwt != null) {
        final clientTypeStr = await localDataSource.getClientType();
        ConfigReader.setClientType(
            EnumHelper.enumFromString(clientTypeStr, ClientType.values));
        ConfigReader.setUserJwt(jwt);
        return Right(true);
      }
      return Right(false);
    } on CacheException {
      return Left(CacheFailure());
    }
  }

  @override
  Future<Either<Failure, void>> logOut() async {
    try {
      return Right(await localDataSource.clearSharedPreferences());
    } on CacheException {
      return Left(CacheFailure());
    }
  }

  @override
  Future<Either<Failure, UserGetEnrollmentFormResponse>>
      getEnrollmentForm() async {
    return await remoteDataSource.getEnrollmentForm();
  }

  @override
  Future<Either<Failure, UserAcceptEnrollmentFormResponse>>
      acceptEnrollmentForm({
    @required String externalId,
    @required User user,
    @required List<LegalDocument> legalDocumentsAccepted,
  }) async {
    try {
      final failureOrAcceptEnrollmentResponse =
          await remoteDataSource.acceptEnrollmentForm(
        externalId: externalId,
        user: user,
        legalDocumentsAccepted: legalDocumentsAccepted,
      );

      if (failureOrAcceptEnrollmentResponse.isRight()) {
        final acceptEnrollmentResponse =
            failureOrAcceptEnrollmentResponse.getOrElse(null);
        final jwt = acceptEnrollmentResponse.sessionToken;
        final clientType = acceptEnrollmentResponse.clientType;

        ConfigReader.setUserJwt(jwt);
        await localDataSource.cacheJwt(jwt);

        ConfigReader.setClientType(clientType);
        await localDataSource
            .cacheClientType(EnumHelper.enumToString(clientType));
      }
      return failureOrAcceptEnrollmentResponse;
    } on ServerException {
      return Left(ServerFailure());
    }
  }

  @override
  Future<Either<Failure, dynamic>> getUserData() async {
    return await remoteDataSource.getUserData();
  }

  @override
  Future<Either<Failure, dynamic>> updateUserData({
    String alias,
    String gender,
    String birthdate,
  }) async {
    return await remoteDataSource.updateUserData(
      alias: alias,
      gender: gender,
      birthdate: birthdate,
    );
  }

}
