import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:dcoupon_sdk_flutter_package/src/config_reader.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/error/failures.dart';
import 'package:dcoupon_sdk_flutter_package/src/core/usecases/usecase.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/entities/dcoupon_listener.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/coupon_activate_coupon.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/coupon_create_user_coupons.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/coupon_deactivate_coupon.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/coupon_get_coupon_detail.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/coupon_get_coupons.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/coupon_get_user_coupons_companies.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/coupon_get_user_coupons_retailers.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/loyalty_add_or_update_loyalty_card.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/loyalty_delete_loyalty_card.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/loyalty_get_loyalty_affiliates.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/loyalty_get_user_loyalty_cards.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/promotion_get_categories.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/promotion_get_companies.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/promotion_get_promotion_detail.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/promotion_get_promotions.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/promotion_get_retailers.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/redemption_get_redemption_code.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/redemption_get_redemption_detail.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/redemption_get_user_redemption_history.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/redemption_get_user_redemption_summary.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/store_get_stores.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/user_get_enrollment_form.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/user_get_user_data.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/user_is_logged_in.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/user_log_in.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/user_log_out.dart';
import 'package:dcoupon_sdk_flutter_package/src/domain/usecases/user_update_user_data.dart';
import 'package:dcoupon_sdk_flutter_package/src/presentation/enrollment_form_app.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/constants.dart';
import 'package:dcoupon_sdk_flutter_package/src/utils/translations.dart';
import 'package:flutter/cupertino.dart';
import 'package:ntp/ntp.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:dcoupon_sdk_flutter_package/src/injection_container.dart' as di;

class Dcoupon {
  BuildContext context;
  Translations translations;
  SharedPreferences prefs;

  Dcoupon({
    BuildContext context,
  }) {
    this.context = context;
  }

  void setContext(BuildContext context) {
    this.context = context;

    _loadTranslations();
  }

  Future<void> loadConfig(String contents) async {
    await di.init();

    try {
      await _getDateOffsetFromNTP();
      await _loadTranslations();
      ConfigReader.init(contents);
      if (!ConfigReader.isSetUp()) {
        throw new Exception(
            'EXCEPTION: Please configure your dcoupon sdk properly');
      }
    } catch (ex) {
      print("dcoupon.loadConfig.catchError: " + ex.toString());
    }
  }

  Future<void> _getDateOffsetFromNTP() async {
    try {
      final offset = await NTP.getNtpOffset().timeout(
            Duration(seconds: 5),
          );
      ConfigReader.setDateOffset(offset);
    } catch (e) {
      print(e);
    }
  }

  Future<void> _loadTranslations() async {
    try {
      var languageCode;
      if (context != null) {
        languageCode = Localizations.localeOf(context).languageCode;
        translations = await Translations.load(Localizations.localeOf(context));
      } else {
        languageCode = 'en';
        translations = await Translations.load(new Locale(languageCode));
      }

      ConfigReader.setLanguageCode(languageCode);
    } catch (ex) {
      print("dcoupon._loadTranslations.catchError: " + ex.toString());
    }
  }

  ///
  /// Log in method.
  ///
  /// @param email      User's email.
  /// @param externalId User's id from your system.
  /// @param alias      User's alias/name.
  /// @param gender     User's gender. Allowed values:'MALE', 'FEMALE', 'OTHER', 'RATHERNOTSAY'. Default 'RATHERNOTSAY'.
  /// @param birthdate  User's date of birth.
  ///
  Future<DcouponListener> logIn(
    String email,
    String externalId, {
    String alias,
    String gender,
    String birthdate,
  }) async {
    final completer = new Completer<DcouponListener>();
    final dcouponListener = DcouponListener();

    if (email == null ||
        email.isEmpty ||
        externalId == null ||
        externalId.isEmpty) {
      dcouponListener.code = HttpStatus.badRequest;
      dcouponListener.message = translations.text("dcoupon_missing_parameters");
      completer.complete(dcouponListener);
      return completer.future;
    }

    final userLogIn = di.sl<UserLogIn>();
    final failureOrUserLoginResponse = await userLogIn(
      UserLogInParams(
        email: email,
        externalId: externalId,
        alias: alias,
        gender: gender,
        birthdate: birthdate,
      ),
    );

    failureOrUserLoginResponse.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
          completer.complete(dcouponListener);
        }
      },
      (userLoginResponse) {
        if (userLoginResponse.legalTermsAccepted) {
          dcouponListener.message = Constants.TRUE;
          dcouponListener.code = HttpStatus.ok;
          completer.complete(dcouponListener);
        } else {
          _startEnrollment(completer, externalId);
        }
      },
    );

    return completer.future;
  }

  void _startEnrollment(
      Completer<DcouponListener> completer, String externalId) async {
    final dcouponListener = DcouponListener();

    final userGetEnrollmentForm = di.sl<UserGetEnrollmentForm>();
    final failureOrUserEnrollmentFormRespone =
        await userGetEnrollmentForm(NoParams());

    failureOrUserEnrollmentFormRespone.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
          completer.complete(dcouponListener);
        }
      },
      (userEnrollmentFormResponse) {
        Navigator.of(context).push(
          PageRouteBuilder(
            opaque: false,
            pageBuilder: (BuildContext context, _, __) {
              return EnrollmentFormApp(
                enrollmentFormAppParams: EnrollmentFormAppParams(
                  translations: translations,
                  completer: completer,
                  externalId: externalId,
                  enrollmentForm:
                      userEnrollmentFormResponse.enrollmentFormModel,
                ),
              );
            },
            settings: RouteSettings(name: EnrollmentFormApp.routeName),
          ),
        );
      },
    );
  }

  /// Returns if the user is logged in dcoupon service.
  Future<bool> isLoggedIn() async {
    final userIsLoggedIn = di.sl<UserIsLoggedIn>();
    final failureOrIsLoggedIn = await userIsLoggedIn(NoParams());

    return failureOrIsLoggedIn.fold(
      (failure) {
        if (failure is CacheFailure) {
          return false;
        }
        return false;
      },
      (isLoggedIn) {
        return isLoggedIn;
      },
    );
  }

  ///Log out the user and clear all the related data.
  Future<void> logOut() async {
    final userLogOut = di.sl<UserLogOut>();
    await userLogOut(NoParams());
    return;
  }

  ///
  /// Get user's data.
  ///
  Future<DcouponListener> getUserData() async {
    final dcouponListener = DcouponListener();

    final userGetUserData = di.sl<UserGetUserData>();
    final failureOrUserDataResponse = await userGetUserData(NoParams());

    return failureOrUserDataResponse.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (userDataResponse) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(userDataResponse);
        return dcouponListener;
      },
    );
  }

  ///
  /// Update user's data.
  ///
  /// @param alias      User's alias/name.
  /// @param gender     User's gender. Allowed values:'MALE', 'FEMALE', 'OTHER', 'RATHERNOTSAY'. Default 'RATHERNOTSAY'.
  /// @param birthdate  User's date of birth.
  ///
  Future<DcouponListener> updateUserData(
    String alias,
    String gender,
    String birthdate,
  ) async {
    final dcouponListener = DcouponListener();

    final userUpdateUserData = di.sl<UserUpdateUserData>();
    final failureOrUpdatedUserData =
        await userUpdateUserData(UserUpdateUserDataParams(
      alias: alias,
      gender: gender,
      birthdate: birthdate,
    ));

    return failureOrUpdatedUserData.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (updatedUserData) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(updatedUserData);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get a list of user's coupons.
  ///
  /// @param start            Index of first store to be returned. Default value 0.
  /// @param limit            Maximum number of items to return. Default value 100.
  /// @param sortBy           How the result set is sorted. Allowed values: 'NEWEST', 'ENDING', 'VALUE', 'DEFAULT'. Default value 'DEFAULT'.
  /// @param companyTokens    Filter user's coupons by this/these company/s.
  /// @param retailerTokens   Filter user's coupons by this/these retailer/s.
  ///
  Future<DcouponListener> getCoupons({
    int offset,
    int limit = 100,
    String sortBy,
    List<String> companyTokens,
    List<String> retailerTokens,
  }) async {
    final dcouponListener = DcouponListener();

    final couponGetCoupons = di.sl<CouponGetCoupons>();
    final failureOrGetCoupons = await couponGetCoupons(CouponGetCouponsParams(
      offset: offset,
      limit: limit,
      sortBy: sortBy,
      companyTokens: companyTokens,
      retailerTokens: retailerTokens,
    ));

    return failureOrGetCoupons.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (getCoupons) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(getCoupons);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get the details of one user's coupon.
  ///
  /// @param couponId The identifier of the user's coupon.
  ///
  Future<DcouponListener> getCouponDetail(
    int couponId,
  ) async {
    final dcouponListener = DcouponListener();

    final couponGetCouponDetail = di.sl<CouponGetCouponDetail>();
    final failureOrCouponDetail =
        await couponGetCouponDetail(CouponGetCouponDetailParams(
      couponId: couponId.toString(),
    ));

    return failureOrCouponDetail.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (couponDetail) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(couponDetail);
        return dcouponListener;
      },
    );
  }

  ///
  /// Set the activated status on an user's coupon.
  ///
  /// @param couponId The identifier of the user's coupon.
  ///
  Future<DcouponListener> activateCoupon(
    int couponId,
  ) async {
    final dcouponListener = DcouponListener();

    final couponActivateCoupon = di.sl<CouponActivateCoupon>();
    final failureOrActivateCoupon =
        await couponActivateCoupon(CouponActivateCouponParams(
      couponId: couponId.toString(),
    ));

    return failureOrActivateCoupon.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (activateCoupon) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(activateCoupon);
        return dcouponListener;
      },
    );
  }

  ///
  /// Set the deactivated status on an user's coupon.
  ///
  /// @param couponId The identifier of the user's coupon.
  ///
  Future<DcouponListener> deactivateCoupon(
    int couponId,
  ) async {
    final dcouponListener = DcouponListener();

    final couponDeactivateCoupon = di.sl<CouponDeactivateCoupon>();
    final failureOrDeactivateCoupon =
        await couponDeactivateCoupon(CouponDeactivateCouponParams(
      couponId: couponId.toString(),
    ));

    return failureOrDeactivateCoupon.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (deactivateCoupon) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(deactivateCoupon);
        return dcouponListener;
      },
    );
  }

  ///
  /// Create multiple user's coupons by promotion tokens.
  ///
  /// @param promotionTokens Promotions tokens that identifies the promotions.
  /// @param latitude        User latitude coordinate.
  /// @param longitude       User longitude coordinate.
  /// @param source          Source from the request.
  /// @param transId         Transaction identifier if the promotion requires it.
  /// @param crmId           The customer relationship management identifier.
  /// @param publisherName   The client publisher name.
  ///
  Future<DcouponListener> createCoupons(
    List<String> promotionTokens, {
    String latitude,
    String longitude,
    String source,
    String transId,
    String crmId,
    String publisherName,
  }) async {
    final dcouponListener = DcouponListener();

    final couponCreateUserCoupons = di.sl<CouponCreateUserCoupons>();
    final failureOrCreateUserCoupons =
        await couponCreateUserCoupons(CouponCreateUserCouponsParams(
      promotionTokens: promotionTokens,
      latitude: latitude,
      longitude: longitude,
      source: source,
      transId: transId,
      crmId: crmId,
      publisherName: publisherName,
    ));

    return failureOrCreateUserCoupons.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (createUserCoupons) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(createUserCoupons);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get user coupons companies
  ///
  Future<DcouponListener> getCouponsCompanies() async {
    final dcouponListener = DcouponListener();

    final couponGetUserCouponsCompanies =
        di.sl<CouponGetUserCouponsCompanies>();
    final failureOrCouponsCompanies =
        await couponGetUserCouponsCompanies(NoParams());

    return failureOrCouponsCompanies.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (couponCompanies) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(couponCompanies);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get user coupons companies
  ///
  Future<DcouponListener> getCouponsRetailers() async {
    final dcouponListener = DcouponListener();

    final couponGetUserCouponsRetailers =
        di.sl<CouponGetUserCouponsRetailers>();
    final failureOrCouponRetailers =
        await couponGetUserCouponsRetailers(NoParams());

    return failureOrCouponRetailers.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (couponRetailers) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(couponRetailers);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get an available user's redemption code.
  ///
  Future<DcouponListener> getRedemptionCode() async {
    final dcouponListener = DcouponListener();

    final redemptionGetRedemptionCode = di.sl<RedemptionGetRedemptionCode>();
    final failureOrRedemptionCode =
        await redemptionGetRedemptionCode(NoParams());

    return failureOrRedemptionCode.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (redemptionCode) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(redemptionCode);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get user's redemption detail.
  ///
  /// @param redemptionId       Redemption identifier.
  /// @param gtmTimeZoneOffset  GTM timezone offset (-12 <= x <=15). Default value 0.
  ///
  Future<DcouponListener> getRedemptionDetail(
    String redemptionId, {
    int gmtTimeZoneOffset = 0,
  }) async {
    final dcouponListener = DcouponListener();

    final redemptionGetRedemptionDetail =
        di.sl<RedemptionGetRedemptionDetail>();
    final failureOrRedemptionDetail = await redemptionGetRedemptionDetail(
      RedemptionGetRedemptionDetailParams(
        redemptionId: redemptionId,
        gmtTimeZoneOffset: gmtTimeZoneOffset,
      ),
    );

    return failureOrRedemptionDetail.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (redemptionDetail) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(redemptionDetail);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get user's redemption summary.
  ///
  /// @param groupBy              Group by: DAY, WEEK, MONTH, YEAR.
  /// @param startDate            Redemption start date: YYYY-MM-DD format.
  /// @param endDate              Redemption end date: YYYY-MM-DD format.
  /// @param gtmTimeZoneOffset    GTM timezone offset (-12 <= x <=15). Default value 0.
  ///
  Future<DcouponListener> getRedemptionSummary(
    String groupBy, {
    String startDate,
    String endDate,
    int gmtTimeZoneOffset = 0,
  }) async {
    final dcouponListener = DcouponListener();

    final redemptionGetUserRedemptionSummary =
        di.sl<RedemptionGetUserRedemptionSummary>();
    final failureOrUserRedemptionSummary =
        await redemptionGetUserRedemptionSummary(
      RedemptionGetUserRedemptionSummaryParams(
        groupBy: groupBy,
        startDate: startDate,
        endDate: endDate,
        gmtTimeZoneOffset: gmtTimeZoneOffset,
      ),
    );

    return failureOrUserRedemptionSummary.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (userRedemptionSummary) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(userRedemptionSummary);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get user's redemption history.
  ///
  /// @param limit                Maximum number of items to return. Default value 0.
  /// @param startDate            Redemption start date: YYYY-MM-DD format.
  /// @param endDate              Redemption end date: YYYY-MM-DD format.
  /// @param gtmTimeZoneOffset    GTM timezone offset (-12 <= x <=15). Default value 0.
  ///
  Future<DcouponListener> getRedemptionHistory({
    int limit,
    int offset,
    String startDate,
    String endDate,
    int gmtTimeZoneOffset = 0,
  }) async {
    final dcouponListener = DcouponListener();

    final redemptionGetUserRedemptionHistory =
        di.sl<RedemptionGetUserRedemptionHistory>();
    final failureOrUserRedemptionHistory =
        await redemptionGetUserRedemptionHistory(
      RedemptionGetUserRedemptionHistoryParams(
        limit: limit,
        offset: offset,
        startDate: startDate,
        endDate: endDate,
        gmtTimeZoneOffset: gmtTimeZoneOffset,
      ),
    );

    return failureOrUserRedemptionHistory.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (userRedemptionHistory) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(userRedemptionHistory);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get loyalty affiliates.
  ///
  Future<DcouponListener> getLoyaltyAffiliates() async {
    final dcouponListener = DcouponListener();

    final loyaltyGetLoyaltyAffiliates = di.sl<LoyaltyGetLoyaltyAffiliates>();
    final failureOrGetLoyaltyAffiliates =
        await loyaltyGetLoyaltyAffiliates(NoParams());

    return failureOrGetLoyaltyAffiliates.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (getLoyaltyAffiliates) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(getLoyaltyAffiliates);
        return dcouponListener;
      },
    );
  }

  ///
  /// Add or update an user's loyalty card.
  ///
  /// @params affiliateApiToken   Affiliate's api token identifier.
  /// @params cardNumber          Card number identifier.
  ///
  Future<DcouponListener> addUpdateLoyaltyCard(
    String affiliateApiToken,
    String cardNumber,
  ) async {
    final dcouponListener = DcouponListener();

    final loyaltyAddOrUpdateLoyaltyCard =
        di.sl<LoyaltyAddOrUpdateLoyaltyCard>();
    final failureOrAddOrUpdateLoyaltyCard =
        await loyaltyAddOrUpdateLoyaltyCard(LoyaltyAddOrUpdateLoyaltyCardParams(
      affiliateApiToken: affiliateApiToken,
      cardNumber: cardNumber,
    ));

    return failureOrAddOrUpdateLoyaltyCard.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (addOrUpdateLoyaltyCard) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(addOrUpdateLoyaltyCard);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get user's loyalty cards.
  ///
  Future<DcouponListener> getLoyaltyCards() async {
    final dcouponListener = DcouponListener();

    final loyaltyGetUserLoyaltyCards = di.sl<LoyaltyGetUserLoyaltyCards>();
    final failureOrGetUserLoyaltyCards =
        await loyaltyGetUserLoyaltyCards(NoParams());

    return failureOrGetUserLoyaltyCards.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (getUserLoyaltyCards) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(getUserLoyaltyCards);
        return dcouponListener;
      },
    );
  }

  ///
  /// Delete an user's loyalty card.
  ///
  /// @params affiliateApiToken   Affiliate's api token identifier.
  /// @params cardNumber          Card number identifier.
  ///
  Future<DcouponListener> deleteLoyaltyCard(
    String affiliateApiToken,
    String cardNumber,
  ) async {
    final dcouponListener = DcouponListener();

    final loyaltyDeleteLoyaltyCard = di.sl<LoyaltyDeleteLoyaltyCard>();
    final failureOrDeleteLoyaltyCard =
        await loyaltyDeleteLoyaltyCard(LoyaltyDeleteLoyaltyCardParams(
      affiliateApiToken: affiliateApiToken,
      cardNumber: cardNumber,
    ));

    return failureOrDeleteLoyaltyCard.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (deleteLoyaltyCard) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(deleteLoyaltyCard);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get categories: categories with active promotions associated to the publisher api token.
  ///
  /// @param publisherName   The client publisher name.
  ///
  Future<DcouponListener> getCategories({
    String publisherName,
  }) async {
    final dcouponListener = DcouponListener();

    final promotionGetCategories = di.sl<PromotionGetCategories>();
    final failureOrGetCategories =
        await promotionGetCategories(PromotionGetCategoriesParams(
      publisherName: publisherName,
    ));

    return failureOrGetCategories.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (getCategories) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(getCategories);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get companies: companies with active promotions associated to the publisher api token.
  ///
  /// @param start Index of first store to be returned.
  /// @param limit Maximum number of items to return.
  /// @param publisherName   The client publisher name.
  ///
  Future<DcouponListener> getCompanies({
    int start,
    int limit,
    String publisherName,
  }) async {
    final dcouponListener = DcouponListener();

    final promotionGetCompanies = di.sl<PromotionGetCompanies>();
    final failureOrGetCompanies =
        await promotionGetCompanies(PromotionGetCompaniesParams(
      start: start,
      limit: limit,
      publisherName: publisherName,
    ));

    return failureOrGetCompanies.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (getCompanies) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(getCompanies);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get retailers: retailers with active promotions associated to the publisher api token.
  ///
  /// @param start Index of first store to be returned.
  /// @param limit Maximum number of items to return.
  /// @param publisherName   The client publisher name.
  ///
  Future<DcouponListener> getRetailers({
    int start,
    int limit,
    String publisherName,
  }) async {
    final dcouponListener = DcouponListener();

    final promotionGetRetailers = di.sl<PromotionGetRetailers>();
    final failureOrGetRetailers =
        await promotionGetRetailers(PromotionGetRetailersParams(
      start: start,
      limit: limit,
      publisherName: publisherName,
    ));

    return failureOrGetRetailers.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (getRetailers) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(getRetailers);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get all active promotions associated to the publisher api token.
  ///
  /// @param start          Index of first store to be returned. Default value 0.
  /// @param limit          Maximum number of items to return. Default value 100.
  /// @param categoryIds    Filter offers from this/these category/ies.
  /// @param storeIds       Filter offers redeemable in those stores form the selected retailer.
  /// @param retailerTokens Filter offers redeemable in this/these retailer/s.
  /// @param companyTokens  Filter offers by this/these company/s.
  /// @param textSearch     Text to search for into promotion name, promotion description and company owner of the promotions.
  /// @param zipcode        Zip code to filter the search.
  /// @param latitude       User latitude coordinate.
  /// @param longitude      User longitude coordinate.
  /// @param radius         Radius to make the search for coupons available.
  /// @param orderBy        How the result set is sorted.
  /// @param publisherName   The client publisher name.
  ///
  Future<DcouponListener> getPromotions({
    int start,
    int limit = 100,
    List<String> categoryIds,
    List<String> storeIds,
    List<String> retailerTokens,
    List<String> companyTokens,
    String textSearch,
    String zipcode,
    String latitude,
    String longitude,
    String radius,
    String orderBy,
    String publisherName,
  }) async {
    final dcouponListener = DcouponListener();

    final promotionGetPromotions = di.sl<PromotionGetPromotions>();
    final failureOrGetPromotions = await promotionGetPromotions(
      PromotionGetPromotionsParams(
        start: start,
        limit: limit,
        categoryIds: categoryIds,
        storeIds: storeIds,
        retailerTokens: retailerTokens,
        companyTokens: companyTokens,
        textSearch: textSearch,
        zipcode: zipcode,
        latitude: latitude,
        longitude: longitude,
        radius: radius,
        orderBy: orderBy,
        publisherName: publisherName,
      ),
    );

    return failureOrGetPromotions.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (getPromotions) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(getPromotions);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get the details of a promotion.
  ///
  /// @param promotionToken The token that identifies this promotion.
  /// @param publisherName   The client publisher name.
  ///
  Future<DcouponListener> getPromotionDetail(
    String promotionToken, {
    String publisherName,
  }) async {
    final dcouponListener = DcouponListener();

    final promotionGetPromotionDetail = di.sl<PromotionGetPromotionDetail>();
    final failureOrGetPromotionDetail =
        await promotionGetPromotionDetail(PromotionGetPromotionDetailParams(
      promotionToken: promotionToken,
      publisherName: publisherName,
    ));

    return failureOrGetPromotionDetail.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (promotionDetail) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(promotionDetail);
        return dcouponListener;
      },
    );
  }

  ///
  /// Get a list of stores associated with a promotion token identifier.
  ///
  /// @param start          Index of first store to be returned. Default value 0.
  /// @param limit          Maximum number of items to return. Default value 10.
  /// @param promotionToken The token that identifies the promotion.
  /// @param retailerTokens Token from the retailer.
  /// @param zipcode        Zip code to filter the search.
  /// @param latitude       User latitude coordinate.
  /// @param longitude      User longitude coordinate.
  /// @param radius         Radius to make the search for coupons available.
  /// @param publisherName   The client publisher name.
  ///
  Future<DcouponListener> getStores(
    String promotionToken, {
    int start,
    int limit = 10,
    List<String> retailerTokens,
    String zipcode,
    String latitude,
    String longitude,
    String radius,
    String publisherName,
  }) async {
    final dcouponListener = DcouponListener();

    final storeGetStores = di.sl<StoreGetStores>();
    final failureOrGetStores = await storeGetStores(
      StoreGetStoresParams(
        promotionToken: promotionToken,
        latitude: latitude,
        limit: limit,
        longitude: longitude,
        publisherName: publisherName,
        radius: radius,
        retailerTokens: retailerTokens,
        start: start,
        zipcode: zipcode,
      ),
    );

    return failureOrGetStores.fold(
      (failure) {
        if (failure is ServerFailure) {
          dcouponListener.code = failure.httpStatus;
          dcouponListener.message = translations.text('dcoupon_error');
        }
        return dcouponListener;
      },
      (getStores) {
        dcouponListener.code = HttpStatus.ok;
        dcouponListener.message = jsonEncode(getStores);
        return dcouponListener;
      },
    );
  }
}
